package de.be4.smvclipse.prolog;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import de.be4.smvclipse.sablecc.node.AAssignConstraintModuleBodyElement;
import de.be4.smvclipse.sablecc.node.ADefineDeclarationModuleBodyElement;
import de.be4.smvclipse.sablecc.node.AIvarDeclaration;
import de.be4.smvclipse.sablecc.node.AIvarDeclarationModuleBodyElement;
import de.be4.smvclipse.sablecc.node.AVarDeclarationModuleBodyElement;
import de.be4.smvclipse.sablecc.node.Node;
import de.be4.smvclipse.sablecc.node.PIdentifier;
import de.be4.smvclipse.sablecc.node.PModuleBodyElement;

public class ModuleElementsContainer {

	private List<Node> variables = new ArrayList<Node>();
	private List<Node> assignments = new ArrayList<Node>();
	private List<Node> definitions = new ArrayList<Node>();
	private List<Node> inputVariables = new ArrayList<Node>();
	
	/**
	 * @param listOfElts List of elements in a module
	 * This method is suppossed to loop through all elements that a module consists of. Each element is put into a
	 * separate list that is suppossed to hold elements of the same type.
	 */
	public ModuleElementsContainer(LinkedList<PModuleBodyElement> listOfElts) {
		for (Object elt : listOfElts) {
			if ( elt instanceof AVarDeclarationModuleBodyElement )
				variables.add((Node)elt);
			else if ( elt instanceof AAssignConstraintModuleBodyElement )
				assignments.add((Node)elt);
			else if ( elt instanceof ADefineDeclarationModuleBodyElement )
				definitions.add((Node)elt);
			else if ( elt instanceof AIvarDeclarationModuleBodyElement )
				inputVariables.add((Node)elt);
		}
	}
	
	@SuppressWarnings("unchecked")
	public Node getInputParameterDeclaration(String name) {
		for ( int i=0; i<inputVariables.size(); i++ ) {
			List iVarList = ((AIvarDeclarationModuleBodyElement)inputVariables.get(i)).getIvarDeclaration();
			for ( int j=0; j<iVarList.size(); j++ ) {
				AIvarDeclaration ivarDecl = (AIvarDeclaration)iVarList.get(j);
				String s = AST2PrologTransformator.extractField(ivarDecl.getIdentifier());
				
				if ( name.equals(s) ) {
					return ivarDecl;
				} else 
					continue;
			}
		}
		
		return null;
	}
	
	public List<Node> getAllInputVariables() {
		return inputVariables;
	}
	
	public List<Node> getAllVariables() {
		return variables;
	}
	
	public List<Node> getAllAssignments() {
		return assignments;
	}
	
	public List<Node> getAllDefinitions() {
		return definitions;
	}
}
